Crate commonware_coding

Crate commonware_coding 

Encode data to enable recovery from a subset of fragments.

Status

Stability varies by primitive. See README for details.

Structs

CodecConfig
The configuration for decoding shard data.
Config
Configuration common to all encoding schemes.
PhasedAsScheme
An adapter that exposes a PhasedScheme through the Scheme trait.
PhasedCheckedShard
A checked shard produced by adapting a phased scheme into Scheme.
ReedSolomon
A SIMD-optimized Reed-Solomon coder that emits chunks that can be proven against a bmt.
Zoda

Enums

PhasedAsSchemeError
Errors returned by the PhasedAsScheme adapter.
ReedSolomonError
Errors that can occur when interacting with the Reed-Solomon coder.
ZodaError

Traits

PhasedScheme
A phased coding interface with separate local and forwarded shard handling.
Scheme
A scheme for encoding data into pieces, and recovering the data from those pieces.
ValidatingScheme
A marker trait indicating that Scheme::check or PhasedScheme::check proves validity of the encoding.