pub struct NoCoding<H> { /* private fields */ }Expand description
A trivial scheme which performs no coding at all.
Instead, each shard contains all of the data.
The commitment is simply a hash of that data. This struct is generic over the choice of commonware_cryptography::Hasher.