pub struct PhasedAsScheme<P>(/* private fields */);Expand description
An adapter that exposes a PhasedScheme through the Scheme trait.
In most cases, this is not the most optimal way to use a PhasedScheme,
or to expose a PhasedScheme as a Scheme for that matter. However,
it can be useful for testing or for usecases where the phased scheme
cannot be used directly.