ValidatingScheme

Trait ValidatingScheme 

pub trait ValidatingScheme { }
A marker trait indicating that Scheme::check or PhasedScheme::check proves validity of the encoding.

In more detail, this means that upon a successful call to Scheme::check, guarantees that the shard results from a valid encoding of the data, and thus, if other participants also call check, then the data is guaranteed to be reconstructable.

Implementors